package com.mortals.xhx.common.formdesign.component;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.formdesign.*;
import com.mortals.xhx.common.pdu.gen.component.ComponentCons;
import com.mortals.xhx.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 动态表格组件
 *
 * @author: zxfei
 * @date: 2021/10/26 10:35
 * @description:
 **/
@Slf4j
public class DesignDynamicTableComponent extends DesignComponent {

    public DesignDynamicTableComponent(String type) {
        super(type);
    }

    @Override
    public ListItem buildDefaultComponent(ComponentCons cons) {
        ListItem listItem = new ListItem();
        listItem.setId(cons.getVal());
        listItem.set_id(cons.getVal());
        listItem.setCompType("dynamicTable");
        listItem.setEle("dynamic-table");
        listItem.setCompName("动态表格");
        listItem.setLayout("dynamicItem");
        listItem.setCompIcon("table");
        listItem.setLabelWidth(0);
        listItem.setShowLabel(false);
        listItem.setConfig(true);
        listItem.setButtonAdd(true);
        listItem.setButtonDel(true);
        listItem.setShowIndex(true);
        listItem.setVisible(true);
        listItem.setMultiCheck(false);
        listItem.setShowSummary(false);
        listItem.setSumText("合计");
        listItem.setSummaryText("元");
        listItem.setMark(new Mark());
        if (!ObjectUtils.isEmpty(cons.getColList())) {
            //动态生成列元素
            List<ColumnsItem> columnsItemList = cons.getColList().stream().map(item -> {
                ColumnsItem columnsItem = new ColumnsItem();
                columnsItem.setId(item.getVal());
                columnsItem.set_id(item.getVal());
                if (item.getColType().equals(ComponentEnum.INPUT.getValue())) {
                    //创建输入框出初始值
                    columnsItem.setCompType("input");
                    columnsItem.setCompName("单行文本");
                    columnsItem.setCompIcon("input");
                    columnsItem.setViewType("text");
                    columnsItem.setEle("el-input");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入" + item.getLabel());
                    columnsItem.setRequired(false);
                    columnsItem.setMaxLength(50);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(24);
                    columnsItem.setWidth("100%");
                    columnsItem.setClearable(true);
                    columnsItem.setDisabled(false);
                    columnsItem.setReadonly(false);
                    columnsItem.setStatus("normal");
                    columnsItem.setPrefixIcon("");
                    columnsItem.setSuffixIcon("");
                    columnsItem.setValue("");
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setRulesType("default");
                    columnsItem.setAppend("");
                    columnsItem.setPrepend("");
                    columnsItem.setLayout("colItem");
                }else if(item.getColType().equals(ComponentEnum.TEXTAREA.getValue())){
                    columnsItem.setCompType("textarea");
                    columnsItem.setEle("el-input");
                    columnsItem.setCompName("多行文本");
                    columnsItem.setCompIcon("textarea");
                    columnsItem.setViewType("text");
                    columnsItem.setType("textarea");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入" + item.getLabel());
                    columnsItem.setRequired(false);
                    columnsItem.setMaxLength(200);
                    columnsItem.setShowWordLimit(true);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(24);
                    columnsItem.setClearable(true);
                    columnsItem.setDisabled(false);
                    columnsItem.setReadonly(false);
                    columnsItem.setRows(4);
                    columnsItem.setStatus("normal");
                    columnsItem.setValue("");     //设置默认值
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setRuleError("该字段不能为空");
                    columnsItem.setRulesType("default");
                }else if(item.getColType().equals(ComponentEnum.SELECT.getValue())){
                    columnsItem.setCompType("select");
                    columnsItem.setCompName(item.getLabel());
                    columnsItem.setCompIcon("select");
                    columnsItem.setEle("el-select");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入"+item.getLabel());
                    columnsItem.setMaxLength(50);
                    columnsItem.setRequired(false);
                    columnsItem.setMultiple(false);
                    columnsItem.setCollapseTags(false);
                    columnsItem.setDisabled(false);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(12);
                    columnsItem.setDataType("static");
                    columnsItem.setClearable(false);
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setLayout("colItem");
                    columnsItem.setRuleError("该字段不能为空");
                    columnsItem.setRulesType("default");

                    initDefaultSelectValue(item, columnsItem);
                }else if(item.getColType().equals(ComponentEnum.RADIO.getValue())){

                    log.info("radio:{}", JSON.toJSONString(item));

                    columnsItem.setCompType("radio");
                    columnsItem.setCompName(item.getLabel());
                    columnsItem.setCompIcon("radio");
                    columnsItem.setEle("el-radio-group");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入"+item.getLabel());
                    columnsItem.setMaxLength(50);
                    columnsItem.setRequired(false);
                    columnsItem.setDisabled(false);
                    columnsItem.setBorder(false);
                    columnsItem.setSize("medium");
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(12);
                    columnsItem.setOptionType("default");
                    columnsItem.setDataType("static");
                    columnsItem.setClearable(false);
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setLayout("colItem");
                    columnsItem.setRuleError("该字段不能为空");
                    initDefaultRadioValue(item, columnsItem);
                }else if(item.getColType().equals(ComponentEnum.CHECKBOX.getValue())){
                    columnsItem.setCompType("checkbox");
                    columnsItem.setCompName(item.getLabel());
                    columnsItem.setCompIcon("checkbox");
                    columnsItem.setEle("el-checkbox-group");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入"+item.getLabel());
                    columnsItem.setMaxLength(50);
                    columnsItem.setRequired(false);
                    columnsItem.setMultiple(false);
                    columnsItem.setCollapseTags(false);
                    columnsItem.setDisabled(false);
                    columnsItem.setBorder(false);
                    columnsItem.setSize("medium");
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(12);
                    columnsItem.setOptionType("default");
                    columnsItem.setDataType("static");
                    columnsItem.setAction("https://www.fastmock.site/mock/51715c0157535b99010bde55f2df33c8/formDesigner/api/options");
                    columnsItem.setClearable(false);
                    columnsItem.setValue("");
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setLayout("colItem");
                    columnsItem.setRuleError("该字段不能为空");
                    columnsItem.setRulesType("default");

                    initDefaultCheckBoxValue(item, columnsItem);

                }else if(item.getColType().equals(ComponentEnum.IMAGE.getValue())){
                    columnsItem.setCompType("upload");
                    columnsItem.setCompName("附件");
                    columnsItem.setCompIcon("att");
                    columnsItem.setEle("el-upload");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setMultiple(false);
                    columnsItem.setRequired(false);
                    columnsItem.setAction(item.getUploadFileUrl());
                    columnsItem.setName("file");
                    columnsItem.setShowFileList(true);
                    columnsItem.setListType("picture-card");
                    columnsItem.setShowTip(false);
                    columnsItem.setTips("点击按钮上传附件");
                    columnsItem.setAccept("jpg,png,jpeg,bmp");
                    columnsItem.setFileSize(10);
                }else if(item.getColType().equals(ComponentEnum.BUTTON.getValue())){
                    columnsItem.setCompType("button");
                    columnsItem.setCompName(item.getLabel());
                    columnsItem.setCompIcon("button");
                    columnsItem.setEle("el-button");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setType("primary");
                    columnsItem.setText("按钮");
                    columnsItem.setPlain(true);
                    columnsItem.setRound(false);
                    columnsItem.setCircle(false);
                    columnsItem.setLoading(false);
                    columnsItem.setLayout("colItem");
                }else if(item.getColType().equals(ComponentEnum.CASCADE.getValue())){
                    columnsItem.setId(item.getVal());
                    columnsItem.set_id(item.getVal());
                    columnsItem.setCompType("cascader");
                    columnsItem.setEle("el-cascader");
                    columnsItem.setCompName("级联选择");
                    columnsItem.setCompIcon("cascader");
                    columnsItem.setViewType("component");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请选择" + item.getLabel());
                    columnsItem.setRequired(false);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(24);

                    columnsItem.setSize("medium");
                    columnsItem.setDisabled(false);
                    columnsItem.setDataType("static");
                    columnsItem.setChinaAreaData("");
                    columnsItem.setAction("");

                    //Props 设置
                    Props props = new Props();
                    props.setClearable(false);
                    props.setSeparator("-");
                    props.setShowAllLevels(true);
                    props.setFilterable(false);

                    Props propsChild = new Props();
                    propsChild.setExpandTrigger("click");
                    propsChild.setMultiple(false);
                    propsChild.setCheckStrictly(false);
                    propsChild.setLabel("label");
                    propsChild.setValue("value");
                    propsChild.setChildren("children");
                    props.setProps(propsChild);
                    columnsItem.setProps(props);

                    //设置options
                    List<OptionsItem> options = new ArrayList<>();
                    OptionsItem optionsItem = new OptionsItem();
                    optionsItem.setValue("zhinan");
                    optionsItem.setLabel("指南");
                    List<ChildrenItem> children = new ArrayList<>();
                    ChildrenItem child = new ChildrenItem();
                    child.setValue("shejiyuanze");
                    child.setLabel("设计原则");
                    children.add(child);
                    optionsItem.setChildren(children);

                    List<ChildrenItem> children1 = new ArrayList<>();
                    ChildrenItem child1 = new ChildrenItem();
                    child1.setValue("yizhi");
                    child1.setLabel("一致");
                    children1.add(child1);
                    child.setChildren(children1);

                    columnsItem.setOptions(options);
                }else if(item.getColType().equals(ComponentEnum.DATE.getValue())){
                    columnsItem.setCompType("date");
                    columnsItem.setEle("el-date-picker");
                    columnsItem.setCompName("日期");
                    columnsItem.setCompIcon("date");
                    columnsItem.setViewType("text");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setType("date");
                    columnsItem.setPlaceholder("请选择" + item.getLabel());
                    columnsItem.setRequired(false);
                    columnsItem.setMaxLength(50);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(12);
                    columnsItem.setValue("");
                    columnsItem.setClearable(true);
                    columnsItem.setDisabled(false);
                    columnsItem.setReadonly(false);
                    columnsItem.setRules(new ArrayList<>());
                    columnsItem.setRangeSeparator(" ");
                    columnsItem.setFormat("yyyy 年 MM 月 dd 日");
                    columnsItem.setValueFormat("yyyy 年 MM 月 dd 日");
                    columnsItem.setStartPlaceholder("开始日期");
                    columnsItem.setEndPlaceholder("结束日期");
                }
                columnsItem.setLayout("colItem");
                return columnsItem;
            }).collect(Collectors.toList());
            listItem.setColumns(columnsItemList);
        }
        return listItem;
    }


    private void initDefaultCheckBoxValue(TableColItem cons, ColumnsItem columnsItem) {
        if (cons.getVal().indexOf("<") != -1) {
            String cks = StringUtils.substringBetween(cons.getVal(), "<", ">");
            if(ObjectUtils.isEmpty(cks))return;
            List<OptionsItem> optionsItemList = StrUtil.splitTrim(cks, "-", -1).stream().map(option -> {
                OptionsItem options = new OptionsItem();
                options.setLabel(option);
                options.setValue(option);
                return options;
            }).collect(Collectors.toList());
            columnsItem.setOptions(optionsItemList);
        }
    }

    private void initDefaultRadioValue(TableColItem cons, ColumnsItem columnsItem) {
        if (cons.getVal().indexOf("<") != -1) {
            String cks = StringUtils.substringBetween(cons.getVal(), "<", ">");
            if(ObjectUtils.isEmpty(cks))return;
            List<OptionsItem> optionsItemList = StrUtil.splitTrim(cks, "-", -1).stream().map(option -> {
                OptionsItem options = new OptionsItem();
                options.setLabel(option);
                options.setValue(option);
                return options;
            }).collect(Collectors.toList());
            columnsItem.setOptions(optionsItemList);
        }
    }


    private void initDefaultSelectValue(TableColItem cons, ColumnsItem columnsItem) {
        //判断如果包含年，月，日，选项框 基础化
        if (cons.getVal().indexOf("年") != -1) {
            //初始化10年
            List<OptionsItem> optionsItems = initYears();
            columnsItem.setOptions(optionsItems);
        } else if (cons.getVal().indexOf("月") != -1) {
            List<OptionsItem> optionsItems = initMonths();
            columnsItem.setOptions(optionsItems);
        } else if (cons.getVal().indexOf("日") != -1) {
            List<OptionsItem> optionsItems = initDays();
            columnsItem.setOptions(optionsItems);
        }

        if (cons.getVal().indexOf("<") != -1) {
            String cks = StringUtils.substringBetween(cons.getVal(), "<", ">");
            if(ObjectUtils.isEmpty(cks))return;
            List<OptionsItem> optionsItemList = StrUtil.splitTrim(cks, "-", -1).stream().map(option -> {
                OptionsItem options = new OptionsItem();
                options.setLabel(option);
                options.setValue(option);
                return options;
            }).collect(Collectors.toList());
            columnsItem.setOptions(optionsItemList);

            // cons.setVal(cons.getVal().substring(0, cons.getVal().indexOf("[")));
            // cons.setLabel(cons.getLabel().substring(0, cons.getLabel().indexOf("[")));

        }
    }

    public List<OptionsItem> initYears() {
        List<OptionsItem> list = new ArrayList<>();
        for (int i = 2020; i < 2030; i++) {
            OptionsItem optionsItem = new OptionsItem();
            optionsItem.setLabel(i + "");
            optionsItem.setValue(i + "");
            list.add(optionsItem);
        }
        return list;
    }

    public List<OptionsItem> initMonths() {
        List<OptionsItem> list = new ArrayList<>();
        for (int i = 1; i <= 12; i++) {
            OptionsItem optionsItem = new OptionsItem();
            optionsItem.setLabel(i + "");
            optionsItem.setValue(i + "");
            list.add(optionsItem);
        }
        return list;
    }

    public List<OptionsItem> initDays() {
        List<OptionsItem> list = new ArrayList<>();
        for (int i = 1; i <= 31; i++) {
            OptionsItem optionsItem = new OptionsItem();
            optionsItem.setLabel(i + "");
            optionsItem.setValue(i + "");
            list.add(optionsItem);
        }
        return list;
    }


    @Override
    public String getType() {
        return ComponentEnum.DYNAMIC_TABLE.getValue();
    }
}
