package com.mortals.xhx.module.user.web;
import com.mortals.framework.annotation.RepeatSubmit;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.BaseEntity;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.utils.BeanUtil;
import com.mortals.framework.utils.ReflectUtils;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.user.model.SearchHistoryEntity;
import com.mortals.xhx.module.user.model.SearchHistoryQuery;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.user.model.UserFillHistoryEntity;
import com.mortals.xhx.module.user.service.UserFillHistoryService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import static com.mortals.framework.ap.SysConstains.*;
import com.mortals.xhx.common.code.*;
/**
*
* 用户填单记录
*
* @author zxfei
* @date 2025-06-08
*/
@RestController
@RequestMapping("user/fill/history")
public class UserFillHistoryController extends BaseCRUDJsonBodyMappingController<UserFillHistoryService,UserFillHistoryEntity,Long> {

    @Autowired
    private ParamService paramService;

    @Value("${server.php:http://192.168.0.98:8090}")
    private String phpServer;


    public UserFillHistoryController(){
        super.setModuleDesc( "用户填单记录");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }

    @PostMapping({"h5/save"})
    @RepeatSubmit
    @UnAuth
    public String saveH5(@RequestBody UserFillHistoryEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        int code = 1;
        String busiDesc = "H5保存" + this.getModuleDesc();
        try {
            this.saveBefore(entity, model, context);
            if (entity.newEntity()) {
                UserFillHistoryEntity initEntity = new UserFillHistoryEntity();
                initEntity.initAttrValue();
                BeanUtils.copyProperties(entity, initEntity, BeanUtil.getNullPropertyNames(entity));
                entity = initEntity;
                busiDesc = "H5新增" + this.getModuleDesc();
                initEntity.setCreateTime(new Date());
                initEntity.setCreateUserId(1L);

                String authtoken = request.getHeader("Authtoken");
                String phpUrl = phpServer + "/wechat/homebase/realInfo";
                JSONObject resp = getWxInfo(phpUrl,authtoken);
                if(resp!=null){
                    initEntity.setOpenId(resp.getString("openId"));
                    initEntity.setMobile(resp.getString("mobile"));
                    initEntity.setIdCard(resp.getString("idCard"));
                }
                this.service.save(initEntity, context);
            } else {
                busiDesc = "H5修改" + this.getModuleDesc();
                entity.setUpdateTime(new Date());
                entity.setUpdateUserId(1L);

                this.service.update(entity, context);
            }

            model.put("id", entity.getId());
            code = this.saveAfter(entity, model, context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var9) {
            this.doException(this.request, busiDesc, model, var9);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var9);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    private JSONObject getWxInfo(String url,String authtoken) {
        String resp = null;
        try {
            Map<String, String> headerMap = new HashMap<>();
            headerMap.put("Authtoken",authtoken);
            Map<String, String> paramsMap = new HashMap<>();
            resp = HttpUtil.doPost(url, headerMap,paramsMap);
            JSONObject jsonObject = JSONObject.parseObject(resp);
            if(jsonObject.getIntValue("code")==1){
                String openId = JSONObject.parseObject(jsonObject.getString("data")).getString("openid");
                String enphone = JSONObject.parseObject(jsonObject.getString("data")).getString("enphone");
                String enidcard = JSONObject.parseObject(jsonObject.getString("data")).getString("enidcard");
//                String idCard = AESUtil.decryptCBC(enidcard,RANDOM);
//                String mobile = AESUtil.decryptCBC(enphone,RANDOM);
                JSONObject result = new JSONObject();
                result.put("openId",openId);
                result.put("idCard",enidcard);
                result.put("mobile",enphone);
                return result;
            }else {
                throw new AppException("用户没有实名信息");
            }
        }catch (Exception e) {
            throw new AppException(e.getMessage());
        }
    }

}